@@ -36,6 +36,7 @@ using namespace QtMobility;
 #include <QContactAddress>
 #include <QContactUrl>
 #include <QContactNickname>
+#include <QContactOrganization>
 
 #include <QMaemo5InformationBox>
 
@@ -70,6 +71,9 @@ void QrSink::clickAction(QWidget *parentWindow) {
     if(type==QrSink::vCard){
         addVCard(barcodeText); //openGeneric(barcodeText); //
 
+    }else if(type==QrSink::iCal){
+            //addiCal(barcodeText);
+
     }else if(type==QrSink::BizCard){
         addMECARD(barcodeText);
 
@@ -155,6 +159,12 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
             barcodeText=barcodeData;
             barcodeInformativeText = barcodeData; // todo, parse and add name of person here
 
+        }else if (barcodeData.indexOf("BEGIN:VCALENDAR",0,Qt::CaseInsensitive)==0){
+            type=this->QrSink::iCal;
+            buttonText.append("Open iCal");
+            barcodeText=barcodeData;
+            barcodeInformativeText = barcodeData; // todo, parse and add name of person here
+
         }else if(barcodeData.indexOf("BIZCARD:",0,Qt::CaseInsensitive)==0){
             type=this->QrSink::BizCard;
             buttonText.append("Open BizCard - may work, fingers crossed");
@@ -633,59 +643,25 @@ void QrSink::addVCard(QString user_data)
     openGeneric(file.fileName());
 
 
-    //file.~QTemporaryFile();
-
-
-    /*QVersitReader reader;
+    /*
+    // this code might actually work now, but best wait for qt-mobility 1.1 + versit
+    QVersitReader reader;
     QBuffer b(user_data.toAscii(), this);
     b.open(QBuffer::ReadOnly);
     reader.setDevice(&b);
+    reader.startReading();
+    reader.waitForFinished();
+    QList<QVersitDocument> inputDocuments = reader.results();
 
-    reader.readAll();
-
-    QVersitDocument inputDocument = reader.result().takeFirst();
     QVersitContactImporter importer;
-    QContact contact = importer.importContact(inputDocument);
+     if (!importer.importDocuments(inputDocuments))
+         return;
+     QList<QContact> contacts = importer.contacts();
+     // Note that the QContacts are not saved yet.
+     // Use QContactManager::saveContacts() for saving if necessary
+    QContactManager::saveContacts(contacts);
+    */
 
-    QContactManager::saveContact(contact); */
-
-
-
-//    g_type_init();
-//    QLibrary *&lib = qgconf_libgconf;
-//    qDebug() << "loading gconf";;
-//    lib = new QLibrary(QLatin1String("gconf-2"));
-//    triedToLoadLibrary = true;
-//    if (lib->load() && lib->resolve("gconf_client_get_default")) {
-//        qDebug() << "load successfull!";
-//    } else {
-//        qDebug() << "Load failed";
-//        lib->unload();
-//        delete lib;
-//        lib = 0;
-//        return;
-//    }
-//
-//    char *vcard = (char*)user_data.constData();
-//    OssoABookContact *contact;
-//    GError *error = NULL;
-//    EBook *book = NULL;
-//    book = osso_abook_system_book_dup_singleton (TRUE, NULL);
-//    g_return_val_if_fail (E_IS_BOOK (book), FALSE);
-//
-//    if (!vcard)
-//        return;
-//
-//    contact = osso_abook_contact_new_from_vcard (NULL, vcard);
-//    g_assert (OSSO_ABOOK_IS_CONTACT (contact));
-//
-//    if (!e_book_add_contact (book, E_CONTACT (contact), &error)) {
-//            g_warning ("%s: %s", G_STRLOC, error->message);
-//            g_assert_not_reached ();
-//    }
-//
-//    lib->unload();
-//    delete lib;
 }
 
 void QrSink::addMECARD(QString user_data)
@@ -731,6 +707,8 @@ void QrSink::addMECARD(QString user_data)
     for(int recordIdx = 0; recordIdx < records.length(); recordIdx++) {
         QString currentRecord = records.at(recordIdx);
 
+        qDebug() << "QrSink::addMECARD() current record=" << currentRecord;
+
         // see what this record encodes
         if(currentRecord.indexOf("N:",0,Qt::CaseSensitive)==0){
             // name
@@ -749,7 +727,7 @@ void QrSink::addMECARD(QString user_data)
             }else{
                 // just a name, no first or last
                 // so what should we do?
-                newContactName.setCustomLabel(nameString);
+                newContactName.setLastName(nameString);
                 qDebug() << "QrSink::addMECARD() found name: " << nameString;
             }
             if(newContact.saveDetail(&newContactName))
@@ -822,9 +800,17 @@ void QrSink::addMECARD(QString user_data)
         }else if(currentRecord.indexOf("BDAY:",0,Qt::CaseSensitive)==0){
             // birthday
             QString dateString(currentRecord.mid(5)); // chop off the BDAY:
-            QString year = dateString.mid(0,4);
-            QString month = dateString.mid(4,2);
-            QString day = dateString.mid(6,2);
+            QString year;
+            QString month;
+            QString day;
+
+            if (dateString.length()==6){
+                year = dateString.mid(0,4);
+                month = dateString.mid(4,2);
+                day = dateString.mid(6,2);
+            }else
+                return;
+
             QDate bdayDate(year.toInt(), month.toInt(), day.toInt());
             QContactBirthday newBDay;
 
@@ -838,26 +824,78 @@ void QrSink::addMECARD(QString user_data)
 
 
 
+        }else if(currentRecord.indexOf("ORG:",0,Qt::CaseSensitive)==0){
+            QString orgString(currentRecord.mid(4)); // chop off the ORG:
+            QContactOrganization newOrg;
+
+            newOrg.setName(orgString);
+
+            qDebug() << "QrSink::addMECARD() found organisation address: " << orgString;
+            if(newContact.saveDetail(&newOrg))
+                qDebug() << "Saved organisation";
+            else
+                qDebug() << "Failed to save organisation";
+
+
         }else if(currentRecord.indexOf("ADR:",0,Qt::CaseSensitive)==0){
             // address
             QString adrString(currentRecord.mid(4)); // chop off the ADR:
             QStringList adrItems = adrString.split(",");
             QContactAddress newAdr;
 
-            newAdr.setPostOfficeBox(adrItems.at(0)); // PO box
-            newAdr.setStreet(adrItems.at(1) + QString(" ") + adrItems.at(2)); // room number + house number
-            newAdr.setLocality(adrItems.at(3)); // city
-            newAdr.setRegion(adrItems.at(4)); // prefecture
-            newAdr.setPostcode(adrItems.at(5)); // zip code
-            newAdr.setCountry(adrItems.at(6)); // country
+            if(adrItems.count()==0){
+                qDebug() << "QrSink::addMECARD(): Unable to parse any address items";
+                return;
 
-            qDebug() << "QrSink::addMECARD() found address (PO box): " << adrItems.at(0);
-            qDebug() << "QrSink::addMECARD() found address (room number): " << adrItems.at(1);
-            qDebug() << "QrSink::addMECARD() found address (house number): " << adrItems.at(2);
-            qDebug() << "QrSink::addMECARD() found address (city): " << adrItems.at(3);
-            qDebug() << "QrSink::addMECARD() found address (prefecture): " << adrItems.at(4);
-            qDebug() << "QrSink::addMECARD() found address (zip code): " << adrItems.at(5);
-            qDebug() << "QrSink::addMECARD() found address (country): " << adrItems.at(6);
+            }else if (adrItems.count()==1){
+                qDebug() << "QrSink::addMECARD(): Parsed 1 address item, assuming this is the street";
+                newAdr.setStreet(adrItems.at(0));
+
+            }else if (adrItems.count()==2){
+                qDebug() << "QrSink::addMECARD(): Parsed 2 address items, assuming this is the street + town";
+                newAdr.setStreet(adrItems.at(0));
+                newAdr.setLocality(adrItems.at(1));
+
+            }else if (adrItems.count()==3){
+                qDebug() << "QrSink::addMECARD(): Parsed 3 address items, assuming this is the street, town, postcode";
+                newAdr.setStreet(adrItems.at(0));
+                newAdr.setLocality(adrItems.at(1));
+                newAdr.setPostcode(adrItems.at(2));
+
+            }else if (adrItems.count()==4){
+                qDebug() << "QrSink::addMECARD(): Parsed 4 address items, assuming this is the street, town, region, postcode";
+                newAdr.setStreet(adrItems.at(0));
+                newAdr.setLocality(adrItems.at(1));
+                newAdr.setRegion(adrItems.at(2));
+                newAdr.setPostcode(adrItems.at(3));
+
+            }else if (adrItems.count()==5){
+                qDebug() << "QrSink::addMECARD(): Parsed 5 address items, assuming this is the street, town, region, postcode, country";
+                newAdr.setStreet(adrItems.at(0));
+                newAdr.setLocality(adrItems.at(1));
+                newAdr.setRegion(adrItems.at(2));
+                newAdr.setPostcode(adrItems.at(3));
+                newAdr.setCountry(adrItems.at(4));
+
+            }else if (adrItems.count()==6){
+                qDebug() << "QrSink::addMECARD(): Parsed 6 address items, assuming these follow the MECARD standard";
+                newAdr.setPostOfficeBox(adrItems.at(0)); // PO box
+                newAdr.setStreet(adrItems.at(1) + QString(" ") + adrItems.at(2)); // room number + house number
+                newAdr.setLocality(adrItems.at(3)); // city
+                newAdr.setRegion(adrItems.at(4)); // prefecture
+                newAdr.setPostcode(adrItems.at(5)); // zip code
+                newAdr.setCountry(adrItems.at(6)); // country
+
+            }else{
+                qDebug() << "QrSink::addMECARD(): Parsed more than the standard 6 address items, using the first 6";
+                newAdr.setPostOfficeBox(adrItems.at(0)); // PO box
+                newAdr.setStreet(adrItems.at(1) + QString(" ") + adrItems.at(2)); // room number + house number
+                newAdr.setLocality(adrItems.at(3)); // city
+                newAdr.setRegion(adrItems.at(4)); // prefecture
+                newAdr.setPostcode(adrItems.at(5)); // zip code
+                newAdr.setCountry(adrItems.at(6)); // country
+
+            }
 
             if(newContact.saveDetail(&newAdr))
                 qDebug() << "Saved address";
@@ -902,7 +940,6 @@ void QrSink::addMECARD(QString user_data)
 
     // we should now see if the contact already exists and if so give the option to replace/merge/add
 
-
     // should we add a signal/slot pair so that plugins can signal their success/lack thereof to the ui and the user?
 
     // make the contact compatible
@@ -919,28 +956,38 @@ void QrSink::addMECARD(QString user_data)
 
     }
 
+}
 
-//    // see if this contact already exists
-//
-//    QContactDetailFilter nameFilter;
-//    nameFilter.setDetailDefinitionName(QContactName::DefinitionName, QContactName::FieldNumber);
-//    nameFilter.setValue(incomingCallNbr);
-//    nameFilter.setMatchFlags(QContactFilter::MatchExactly);
-//
-//    QList<QContactLocalId> matchingContacts = cm->contacts(nameFilter);
-//    if (matchingContacts.size() == 0) {
-//        qDebug() << "Incoming call from unknown contact (" << incomingCallNbr << ")";
-//    } else {
-//        QContact match = cm->contact(matchingContacts.at(0));
-//        qDebug() << "Incoming call from"
-//                 << match.displayLabel()
-//                 << "(" << incomingCallNbr << ")";
-//    }
-//
-//
+
+
+
+void QrSink::addiCal(QString user_data)
+{
+
+    /*
+    // wait for qt-mobility 1.1 + versit
+    #include <QVersitOrganizerImporter>
+    #include <QOrganizerItem>
+    #include <QOrganizerManager>
+
+    QVersitReader reader;
+    QBuffer b(user_data.toAscii(), this);
+    b.open(QBuffer::ReadOnly);
+    reader.setDevice(&b);
+    reader.startReading();
+    reader.waitForFinished();
+    QList<QVersitDocument> inputDocuments = reader.results();
+
+    QVersitOrganizerImporter importer;
+    if (!importer.importDocuments(inputDocuments))
+         return;
+    QList<QOrganizerItem> items = importer.items();
+    QOrganizerManager::saveContacts(items);
+    */
 
 
 }
 
 
 
+
