@@ -24,7 +24,8 @@ public:
         Geo,
         YouTube,
         Install,
-        Ovi
+        Ovi,
+        iCal
     };
     // inherited virtuals
     QrSink(const PluginInterface *interface);
@@ -41,6 +42,7 @@ private slots:
 private:
     void openGeneric(QString uri);
     void addVCard(QString user_data);
+    void addiCal(QString user_data);
     void openInstallFile(QString data);
     void openBrowser(QString uri);
     void addMECARD(QString user_data);
